<!DOCTYPE html>
<html>
<head>
<style>
    body{font-family: arial; background: #f8f8f8; color:#345;}
    .box{border: 2px solid rgba(0,0,0,0.2); padding: 1em; margin-bottom: 1em;}
    .ok{color:#383; background:#cfd;}
    .error{color:#833; background:#fdd;}
</style>
</head>
<body>

<h1>CI Server :: Config Check</h1>

<?php
/* ######################################################################

  IML DEPLOYMENT

  CONFIG CHECKER for new setups

  ---------------------------------------------------------------------
  2017-12-01  Axel <axel.hahn@iml.unibe.ch>
  ###################################################################### */


// ----------------------------------------------------------------------
// functions
// ----------------------------------------------------------------------

/**
 * Get html code to draw a colored box
 * 
 * @param string  $s       message
 * @param string  $sClass  css class; one of ok|info|error
 * @return string
 */
function drawbox(string $s, string $sClass): string
{
    return "<div class=\"box $sClass\">$s</div>";
}

/**
 * Show an alert box with error message
 * 
 * @param string $s  message to show
 * @return string
 */
function showerror($s)
{
    return drawbox($s, 'error');
}

/**
 * Show green box with ok message
 * 
 * @param string $s  message to show
 * @return string
 */
function showok($s)
{
    return drawbox($s, 'ok');
}

/**
 * Check a directory if it exists and is writtable
 * 
 * @global array  $aErrors  found errors
 * 
 * @param string  $sDir     directory to check
 * @param string  $sKey     key in config where this directory is defined (for error output only)
 * @return string
 */
function checkdir(string $sDir, string $sKey = ''): string
{
    global $aErrors;
    $sReturn = '';

    $sReturn.="Check directory [$sDir] ";
    if (!is_dir($sDir)) {
        $sReturn.='<span class="error">does not exist</span><br>';
        $aErrors[] = "* \$aConfig['$sKey'] points to a non existing directory ($sDir).\n";
    } else {
        if (!is_writable($sDir)) {
            $sReturn.='<span class="error">not writable</span><br>';
            $aErrors[] = "* \$aConfig['$sKey'] = $sDir is NOT writable.\n";
        } else {
            $sReturn.='<span class="ok">OK</span> exists and is writable<br>';
        }
    }
    return $sReturn;
}

/**
 * Check if a php module was found
 * 
 * @global array  $aErrors  found errors
 * 
 * @param  array  $aRequiredMods  array with needed php modules; default: empty = list all current php modules
 * @return string
 */
function checkModule($aRequiredMods = []): string
{
    global $aErrors;
    $sReturn = '';

    $aAllMods = get_loaded_extensions(false);
    asort($aAllMods);
    if (!count($aRequiredMods)) {
        $sReturn.= '<strong>List of all installes php modules</strong><br>';
        foreach ($aAllMods as $sMod) {
            $sReturn.= "$sMod<br>";
        }
    } else {
        foreach ($aRequiredMods as $sMod) {
            $sReturn.="$sMod - ";
            if (!array_search($sMod, $aAllMods) === false) {
                $sReturn.='<span class="ok">OK</span> installed';
            } else {
                $sReturn.='<span class="error">does not exist</span>';
                $aErrors[] = "* php module $sMod was not found.\n";
            }
            $sReturn.='<br>';
        }
    }
    return $sReturn;
}

/**
 * Check if a command can be executed
 * 
 * @global array  $aErrors  found errors
 * 
 * @param  array  $aCommands  array with key=command + value=description
 * @return string
 */
function checkCommands(array $aCommands): string
{
    global $aErrors;
    $sReturn = '';

    foreach ($aCommands as $sCommand => $sDescription) {
        $sReturn.='command [' . $sCommand . '] (' . $sDescription . ') - ';

        $aOut=[];
        exec("$sCommand 2>&1", $aOut, $iReturn) .  " ";
        $sReturn.= implode("\n", $aOut) . "\n";

        if ($iReturn === 0) {
            $sReturn.='<span class="ok">OK</span> ';
        } else {
            $sReturn.='<span class="error">does not exist</span>';
            $aErrors[] = "* command $sCommand was not found or failed.\n";
        }
        $sReturn.='<br>';
    }
    return $sReturn;
}

// ----------------------------------------------------------------------
// 
// main
// 
// ----------------------------------------------------------------------

$sOut="";

$aErrors = [];
include("../config/inc_projects_config.php");
if (!isset($aConfig) || !is_array($aConfig)) {
    // echo showerror("\$aConfig does not exist. The config was not included before including " . __FILE__ . " in the request/ script.");
    $aErrors[] = "* \$aConfig does not exist. The config was not included before including " . __FILE__ . " in the request/ script.";
} else {

    // ----------------------------------------------------------------------
    $sOut.='<h2>Check keys with directories</h2>';
    foreach ([ // 'appRootDir',
        'configDir', 'workDir', 'dataDir', 'buildDir', 'buildDefaultsDir', 'packageDir', 'archiveDir', 'tmpDir', ] as $sKey) {
        $sOut.=checkdir($aConfig[$sKey], $sKey);
    }

    $sOut.='<h2>Check subdirs of key dataDir</h2>';
    foreach ([$aConfig['dataDir'] . '/database', $aConfig['dataDir'] . '/projects', $aConfig['dataDir'] . '/sshkeys', ] as $sDir2Check) {
        $sOut.=checkdir($sDir2Check, 'dataDir + [subdir]');
    }

    // ----------------------------------------------------------------------
    $sOut.='<h2>Check keys</h2>';
    // check required keys in the config
    foreach (['auth', 'build', 'lang', 'phases', 'projects', ] as $sKey) {
        $sOut.="Key [$sKey] ";
        if (!array_key_exists($sKey, $aConfig)) {
            $sOut.="<span class=\"error\">failed</span> missing key [$sKey] in config<br>";
            $aErrors[] = "* missing key [$sKey] in config\n";
        } else {
            $sOut.='<span class="ok">OK</span> exists<br>';
        }
    }
}

$sOut.='<h2>Check required PHP modules</h2>';
$sOut.='PHP version: ' . PHP_VERSION . '<br>'
    . '<br>';
$sOut.=checkModule([
    'PDO',
    'curl',
    'json',
    'ldap',
    'pdo_sqlite'
]);

$sOut.='<h2>Check executable commands</h2>'
    . '<p>remark: if the apache user has no login shell then all commands will fail.</p>';
$sOut.='<h3>basic tools</h3>';
$sOut.=checkCommands([
    'which bash' => 'shell interpreter to execute hook scripts',
    'which cat' => 'show content of files',
    'which cp' => 'copy files',
    'which ln' => 'create softlinks for version handling',
    'which ls' => 'list files',
    'which mkdir' => 'create working directories',
    'which mv' => 'move files',
    'which ps' => 'process list',
    'which rm' => 'remove files and directories',
]);
$sOut.='<h3>More tools</h3>';
$sOut.=checkCommands([
    'which ssh' => 'connect with ssh for remote execution',
    'which rsync' => 'sync packages to puppet master',
    'which git' => 'connect to git repositories',
]);

$sOut.='<h3>tools for IML projects</h3>';
$sOut.=checkCommands([
    'which uglifyjs' => 'compress js',
    'which yui-compressor' => 'compress js, css',
    'which nodejs' => 'Nodejs - Javascript v8',
    'which yarn' => 'nodejs package manager',
]);




if (count($aErrors)) {
    echo showerror("ERRORS were found:<br>" . implode("<br>\n", $aErrors));
} else {
    echo showok('OK, check was successfully finished.');
}
echo '<h2>Result</h2>';
echo $sOut;
// do not enable - it shows passwords...
// echo 'DEBUG: <pre>' . print_r($aConfig, 1) . '</pre>';

?>
</body>
</html>
